#version 150

#moj_import <light.glsl>
#moj_import <fog.glsl>
#moj_import <shader_utils.glsl>

in vec3 Position;
in vec4 Color;
in vec2 UV0;
in ivec2 UV2;
in vec3 Normal;

uniform sampler2D Sampler2;
uniform sampler2D Sampler0;

uniform mat4 ModelViewMat;
uniform mat4 ProjMat;
uniform vec3 ChunkOffset;
uniform int FogShape;
uniform float GameTime;

out float vertexDistance;
out vec4 vertexColor;
out vec4 lightmapColor;
out vec2 texCoord0;
out vec4 normal;
out float object;

#define pi 3.1415926535897932

void main() {
    vec3 pos = Position + ChunkOffset;
    vec3 position = (Position / 2.0) * pi;
    float animation = GameTime * 4000.0;

    float xs = 0.0;
    float ys = 0.0;
    float zs = 0.0;
	float alpha = texture(Sampler0, UV0).a * 255.0;

    if (alpha == 1.0 || alpha == 252.0) { // Most plants like grass and flowers use this
        xs = sin(position.x + animation);
        zs = cos(position.z + position.y + animation);
    } else if (alpha == 2.0) { // Used for the edges of multi-blocks, like the top block of tall grass or the bottom block of twisting vines
        xs = sin(position.x + position.y + animation) * 2.0;
        zs = cos(position.z + position.y + animation) * 2.0;
        
    } else if (alpha == 3.0) { // Used for spore blossoms' special animation
        xs = sin(position.x + position.y + animation);
        zs = cos(position.z + position.y + animation);
        ys = sin(position.y + (animation / 1.5)) / 9.0;
        
    }

    gl_Position = ProjMat * ModelViewMat * (vec4(pos, 1.0) + vec4(xs / 32.0, ys / 24.0, zs / 32.0, 0.0));

    vertexDistance = fog_distance(ModelViewMat, pos, FogShape);
    vertexColor = getCherryBiomeColor(Color);
    texCoord0 = UV0;
    lightmapColor = minecraft_sample_lightmap(Sampler2, UV2);
    normal = ProjMat * ModelViewMat * vec4(Normal, 0.0);
}
